/**
 * SEO component that queries for data with
 *  Gatsby's useStaticQuery React hook
 *
 * See: https://www.gatsbyjs.org/docs/use-static-query/
 */

import React from 'react';
import PropTypes from 'prop-types';
import Helmet from 'react-helmet';
import { useLocation } from '@gatsbyjs/reach-router';
import { useSiteMetadata } from '../utils/hooks';

export function SEO({
  description,
  meta,
  title,
  shortTitle,
  shortDescription,
  url,
  link,
  lang,
  thumbnailImage // can be imported image object or  full image Url
}) {
  const location = useLocation();

  const siteMetadata = useSiteMetadata();

  // If locale not specified in siteMetadata, use "en"
  const defaultLang = siteMetadata.locales?.default || 'en';
  const pageLang = lang || defaultLang;

  const metaDescription = description || siteMetadata.description;
  // eslint-disable-next-line operator-linebreak
  const metaShortDescription =
    shortDescription || siteMetadata.shortDescription;
  const metaTitle = title || siteMetadata.title;
  const metaShortTitle = shortTitle || siteMetadata.shortTitle;
  const metaUrl = url || siteMetadata.url + location.pathname;

  //? the og image should be absolute url https://stackoverflow.com/questions/9858577/open-graph-can-resolve-relative-url/9858694#9858694

  const userDefinedThumbnailImage = thumbnailImage
    ? thumbnailImage?.startsWith('data:') ||
      thumbnailImage?.startsWith('https://') ||
      thumbnailImage?.startsWith('http://')
      ? thumbnailImage
      : siteMetadata.url + thumbnailImage
    : null;

  const metaImage =
    userDefinedThumbnailImage ||
    siteMetadata.url + siteMetadata.thumbnail?.images.fallback.src;

  const defaultAlternateLinks = [
    {
      rel: 'alternate',
      hreflang: defaultLang,
      href: metaUrl
    },
    {
      rel: 'alternate',
      hreflang: 'x-default',
      href: metaUrl
    }
  ];

  return (
    <Helmet
      htmlAttributes={{
        lang: pageLang
      }}
      link={[
        // If link exists, the intl SEO component will override the links
        ...(link ? [] : defaultAlternateLinks),
        {
          rel: 'image_src',
          type: 'image/png',
          href: siteMetadata.thumbnail?.images.fallback.src
        },
        {
          rel: 'icon',
          type: 'image/png',
          sizes: '32x32',
          href: siteMetadata.favicon32?.src
        },
        {
          rel: 'icon',
          type: 'image/png',
          sizes: '16x16',
          href: siteMetadata.favicon16?.src
        },
        {
          rel: 'shortcut icon',
          type: 'image/png',
          href: siteMetadata.favicon64?.src
        }
      ].concat(link)}
      meta={[
        {
          name: 'description',
          content: metaDescription
        },
        {
          property: 'og:title',
          content: metaShortTitle
        },
        {
          property: 'og:description',
          content: metaShortDescription
        },
        {
          property: 'og:url',
          content: metaUrl
        },
        {
          property: 'og:image',
          content: metaImage
        },
        {
          property: 'og:type',
          content: 'website'
        },
        {
          name: 'twitter:card',
          content: 'summary'
        },
        {
          name: 'twitter:creator',
          content: siteMetadata.author
        },
        {
          name: 'twitter:title',
          content: title
        },
        {
          name: 'twitter:description',
          content: metaDescription
        },
        {
          name: 'twitter:url',
          content: metaUrl
        },
        {
          name: 'twitter:image',
          content: metaImage
        }
      ].concat(meta)}
      title={metaTitle}
      titleTemplate={`%s | ${siteMetadata.title}`}
    />
  );
}

SEO.defaultProps = {
  meta: [],
  description: '',
  shortTitle: null,
  shortDescription: null,
  url: null,
  link: [],
  lang: null,
  thumbnailImage: null
};

SEO.propTypes = {
  description: PropTypes.string,
  lang: PropTypes.string,
  link: PropTypes.arrayOf(PropTypes.object),
  meta: PropTypes.arrayOf(PropTypes.object),
  shortDescription: PropTypes.string,
  shortTitle: PropTypes.string,
  thumbnailImage: PropTypes.string,
  title: PropTypes.string.isRequired,
  url: PropTypes.string
};
